import time

from sklearn_compat.utils._user_interface import _print_elapsed_time


def test_print_elapsed_time():
    """Check that we can import `_print_elapsed_time` from the right module.

    This change has been done in scikit-learn 1.5.
    """
    import sys
    from io import StringIO

    stdout = StringIO()
    sys.stdout = stdout
    with _print_elapsed_time("sklearn_compat", "testing"):
        time.sleep(0.1)
    sys.stdout = sys.__stdout__
    output = stdout.getvalue()
    assert output.startswith("[sklearn_compat] .....")



from sklearn_compat._sklearn_compat import (
    _check_feature_names,  # noqa: F401
    _check_n_features,  # noqa: F401
    _is_fitted,  # noqa: F401
    _to_object_array,  # noqa: F401
    validate_data,  # noqa: F401
)



from sklearn.base import BaseEstimator

from sklearn_compat.utils._tags import (
    get_tags,
)


def test_get_tags():
    class MyEstimator(BaseEstimator):
        def _more_tags(self):
            return {"requires_fit": False}

        def __sklearn_tags__(self):
            tags = super().__sklearn_tags__()
            tags.requires_fit = False
            return tags

    tags = get_tags(MyEstimator())
    assert not tags.requires_fit



import numpy as np

from sklearn_compat.utils._mask import axis0_safe_slice, indices_to_mask, safe_mask


def test_safe_mask():
    data = np.arange(1, 6).reshape(-1, 1)
    condition = [False, True, True, False, True]
    mask = safe_mask(data, condition)
    np.testing.assert_array_equal(data[mask], np.array([[2], [3], [5]]))


def test_axis0_safe_slice():
    X = np.random.randn(5, 3)
    mask = np.array([True, False, True, False, True])
    result = axis0_safe_slice(X, mask, X.shape[0])
    np.testing.assert_array_equal(result, X[mask])


def test_indices_to_mask():
    indices = [1, 2, 3, 4]
    mask = indices_to_mask(indices, 5)
    np.testing.assert_array_equal(mask, np.array([False, True, True, True, True]))



import numpy as np

from sklearn_compat.utils.extmath import _approximate_mode, safe_sqr


def test__approximate_mode():
    result = _approximate_mode(class_counts=np.array([4, 2]), n_draws=3, rng=0)
    np.testing.assert_array_equal(result, np.array([2, 1]))


def test_safe_sqr():
    result = safe_sqr(np.array([1, 2, 3]))
    np.testing.assert_array_equal(result, np.array([1, 4, 9]))



import numpy as np
import pytest

from sklearn_compat.utils._indexing import (
    _determine_key_type,
    _get_column_indices,
    _safe_indexing,
    resample,
    shuffle,
)


def test__determine_key_type():
    assert _determine_key_type(np.arange(10)) == "int"


def test__safe_indexing():
    array = np.arange(10).reshape(2, 5)
    np.testing.assert_allclose(_safe_indexing(array, 1, axis=1), array[:, 1])


def test__get_column_indices():
    pd = pytest.importorskip("pandas")
    df = pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
    assert np.array_equal(_get_column_indices(df, key="b"), [1])


def test_resample():
    array = np.arange(10)
    resampled_array = resample(array, n_samples=20, replace=True, random_state=0)
    assert len(resampled_array) == 20


def test_shuffle():
    array = np.arange(10)
    shuffled_array = shuffle(array, random_state=0)
    assert len(shuffled_array) == len(array)



from sklearn_compat.utils._chunking import (
    chunk_generator,
    gen_batches,
    gen_even_slices,
    get_chunk_n_rows,
)


def test_chunk_generator():
    gen_chunk = chunk_generator(range(10), 3)
    assert len(next(gen_chunk)) == 3


def test_gen_batches():
    batches = list(gen_batches(7, 3))
    assert batches == [slice(0, 3, None), slice(3, 6, None), slice(6, 7, None)]


def test_gen_even_slices():
    batches = list(gen_even_slices(10, 1))
    assert batches == [slice(0, 10, None)]


def test_get_chunk_n_rows():
    assert get_chunk_n_rows(10) == 107374182



from sklearn_compat.utils._missing import is_pandas_na, is_scalar_nan


def test_is_scalar_nan():
    assert is_scalar_nan(float("nan"))


def test_is_pandas_na():
    assert not is_pandas_na(float("nan"))
